unit UBAGlobals;

{$OPTIMIZATION OFF}

{.$define debug}

interface

uses
  Classes, ORNet, uConst, ORFn, Sysutils, Dialogs, Windows;

type

 {Problem List Record Used To Add New DX From SignOrders Form }
 TBAPLRec = class(TObject)
  constructor Create(PLlist:TStringList);
   function BuildBAProblemRecHDR:TStringList;
   function BuildBAProblemRecDTL(PassList: TStringList;DXSection: string; DXCode:string): TStringList;
   function FMToDateTime(FMDateTime: string): TDateTime;
   function SaveProblemRec(BAProblemList:TStringList):Boolean;
 end;

 {patient qualifiers}
 TBAPLPt=class(TObject)
   PtVAMC:string;
   PtDead:string;
   PtBid:string;
   PtServiceConnected:boolean;
   PtAgentOrange:boolean;
   PtRadiation:boolean;
   PtEnvironmental:boolean;
   PtHNC:boolean;
   PtMST:boolean;
   constructor Create(Alist:TStringList);
   function GetGMPDFN(dfn:string;name:String):string;
 public
   function InitPt(const PatientDFN: string): TStrings ;
   procedure LoadPatientParams(AList:TstringList);

 end;

  TBAGlobals = class(TObject)
  private
     FOrderNum: string;
    protected
  public
     constructor Create;
  published
     property OrderNum: string read FOrderNum write FOrderNum;
    procedure AddBAPCEDiag(DiagStr:string);
     procedure ClearBAPCEDiagList;
     procedure SaveBillingData;
  end;

 TBADxRecord = class(TObject)
     FExistingRecordID: string;
     FOrderID: string;
     FBADxCode: string; //Primary Dx
     FBASecDx1: string; //Secondary Dx 1
     FBASecDx2: string; //Secondary Dx 2
     FBASecDx3: string; //Secondary Dx 3
  end;
  TDxArray = array[0..9] of TBADxRecord;

function tempDxNodeExists(thisOrderID: string) : boolean;
function GetDxNodeIndex(thisOrderID: string) : smallint;
function DiagnosesMatch(var List1: TStringList; var List2: TStringList) : boolean;
function CountSelectedOrders(const Caller: smallint) : smallint;
function CompareOrderDx(const Caller: smallint) : boolean;
procedure GetBADxListForOrder(var thisRetVal: TBADxRecord; thisOrderID: string);
procedure DestroyDxList;
procedure SetBADxList;
procedure SimpleAddTempDxList(thisOrderID: string);
procedure SetBADxListForOrder(thisRec: TBADxRecord; thisOrderID: string);
function AllSelectedDxBlank(const Caller: smallint) : boolean;
function SecondaryDxFull(thisOrderID: string) : boolean;
procedure AddSecondaryDx(thisOrderID: string; thisDxCode: string);
procedure InitializeNewDxRec(var thisDxRec:  TBADxRecord);

function OrdersHaveDx:boolean;

procedure SetBillingAwareSwitch;
function BADxEntered:boolean;

function AttachDxToOrderList:TStringList;
procedure SaveBillingDxEntered;  // save dx enteries regardless of being mandatory....
                                  //This will only be executed if bypass switch is set...

var
  BAGlobals         : TBAGlobals;
  BAPLPt            : TBAPLPt;
  BAPLRec           : TBAPLRec;
  BAProbFile        : TStringList;
  PLlist            : TStringList;
  BADiagnosisList   : TStringList;
  BALocation        : integer;
  BAPCEDiagList     : TStringList;
  DxList            : TList;
  tempDxList        : TList;
  globalDxRec       : TBADxRecord;
  BAOrderList       : TStringList;
  OrderReviewList   : TList;
  ChangeItemOrderNum: PChar;
  OrderIDList       : TStrings;
  BAOrderID         : string;
  BASwitchON        : boolean;

 // Used to display Dx's on fordersSign and fReview grids
  Dx1               : string ;
  Dx2               : string ;
  Dx3               : string ;
  Dx4               : string ;


implementation

uses fBALocalDiagnoses, fOrdersSign, fReview, rOrders, uCore, rCore, rPCE,uPCE,
     UBAConst, UBAMessages;

function AllSelectedDxBlank(const Caller: smallint) : boolean;
var
  i: smallint;
  selectedOrderID: string;
begin
  Result := true;

  case Caller of
     F_ORDERS_SIGN: begin
                    for i := 0 to fOrdersSign.frmSignOrders.clstOrders.Items.Count-1 do
                        if (frmSignOrders.clstOrders.Selected[i]) then
                           begin
                           selectedOrderID :=  TOrder(fOrdersSign.frmSignOrders.clstOrders.Items.Objects[i]).ID;
                           if (tempDxNodeExists(selectedOrderID)) then
                                Result := false;
                           end;
                    end;
     F_REVIEW:      begin
                    for i := 0 to fReview.frmReview.lstReview.Items.Count-1 do
                        if (fReview.frmReview.lstReview.Selected[i]) then
                           begin
                           selectedOrderID :=  TOrder(fReview.frmReview.lstReview.Items.Objects[i]).ID;
                           if tempDxNodeExists(selectedOrderID) then
                                Result := false;
                           end;
                    end;
  end; //case
end;

function GetDxNodeIndex(thisOrderID: string) : smallint;
var
  i: integer;
  thisRec: TBADxRecord;
begin
  Result := 0;
  thisRec := TBADxRecord.Create;
   if Assigned(tempDxList) then
        for i := 0 to (tempDxList.Count - 1) do
           begin
            thisRec := TBADxRecord(tempDxList.Items[i]);
            if Assigned(thisRec) then
              if (thisRec.FOrderID = thisOrderID) then
                 Result := i;
            end;

end;

function DiagnosesMatch(var List1: TStringList; var List2: TStringList) : boolean;
var
  i: smallint;
begin
  Result := false;

  // If the number of Dx's in the lists differs, then bail
  if (List1.Count <> List2.Count) then
     begin
     Result := false;
     Exit;
     end;

  List1.Sort;
  List2.Sort;

  for i := 0 to (List1.Count - 1) do
     if (List1.Strings[i] <> List2.Strings[i]) then
        Result := false
     else
        Result := true;
end;

function CountSelectedOrders(const Caller: smallint) : smallint;
var
  selectedOrders: smallint;
  var i: integer;
begin
  Result := 0;
  selectedOrders := 0;
  
  // How many orders selected?
  case Caller of
     F_ORDERS_SIGN: begin
                    for i := 0 to (fOrdersSign.frmSignOrders.clstOrders.Items.Count-1) do
                        if (fOrdersSign.frmSignOrders.clstOrders.Selected[i]) then
                             Inc(selectedOrders);
                    end;
     F_REVIEW:      begin
                    for i := 0 to (fReview.frmReview.lstReview.Items.Count-1) do
                        if (fReview.frmReview.lstReview.Selected[i]) then
                             Inc(selectedOrders);
                    end;
  end; //case

  Result := selectedOrders;

end;

function CompareOrderDx(const Caller: smallint) : boolean;
var
  i: integer;
  firstSelectedID: string;
  thisOrderID: string;
  firstDxRec: TBADxRecord;
  compareDxRec: TBADxRecord;
  thisStringList: TStringList;
  thatStringList: TStringList;
  //selectedOrders: smallint;
begin
  Result := false;
  firstSelectedID := '';
  firstDxRec := nil;
  firstDxRec := TBADxRecord.Create;
  thisStringList := TStringList.Create;
  thisStringList.Clear;
  thatStringList := TStringList.Create;
  thatStringList.Clear;

  case Caller of
     F_ORDERS_SIGN: begin
                    for i := 0 to (fOrdersSign.frmSignOrders.clstOrders.Items.Count-1) do
                        if (fOrdersSign.frmSignOrders.clstOrders.Selected[i]) then
                           begin
                           firstSelectedID := TChangeItem(fOrdersSign.frmSignOrders.clstOrders.Items.Objects[i]).ID;
                           Break;
                           end;
                    end;
     F_REVIEW:      begin
                    for i := 0 to (fReview.frmReview.lstReview.Items.Count-1) do
                        if (fReview.frmReview.lstReview.Selected[i]) then
                           begin
                           firstSelectedID := TChangeItem(fReview.frmReview.lstReview.Items.Objects[i]).ID;
                           Break;
                           end;
                    end;
  end; //case

  firstDxRec := TBADxRecord.Create;
  InitializeNewDxRec(firstDxRec);
  GetBADxListForOrder(firstDxRec, firstSelectedID);

  // first string to compare
  thisStringList.Add(firstDxRec.FBADxCode);
  thisStringList.Add(firstDxRec.FBASecDx1);
  thisStringList.Add(firstDxRec.FBASecDx2);
  thisStringList.Add(firstDxRec.FBASecDx3);

  case Caller of
     F_ORDERS_SIGN: begin
                    for i := 0 to fOrdersSign.frmSignOrders.clstOrders.Items.Count-1 do
                        if (fOrdersSign.frmSignOrders.clstOrders.Selected[i]) then
                           begin
                           thisOrderID := TChangeItem(fOrdersSign.frmSignOrders.clstOrders.Items.Objects[i]).ID;
                           // If order ID is same as the first selected order, then skip it
                           if thisOrderID = firstSelectedID then
                                Continue
                           else
                               begin
                              compareDxRec := TBADxRecord.Create;
                               InitializeNewDxRec(compareDxRec);
                               GetBADxListForOrder(compareDxRec, thisOrderID);

                               thatStringList.Add(compareDxRec.FBADxCode);
                               thatStringList.Add(compareDxRec.FBASecDx1);
                               thatStringList.Add(compareDxRec.FBASecDx2);
                               thatStringList.Add(compareDxRec.FBASecDx3);

                               if DiagnosesMatch(thisStringList, thatStringList) then
                                   Result := true;
                               end;
                           end;
                    end;
     F_REVIEW:      begin
                    for i := 0 to fReview.frmReview.lstReview.Items.Count-1 do
                        if (fReview.frmReview.lstReview.Selected[i]) then
                           begin
                           thisOrderID := TChangeItem(fReview.frmReview.lstReview.Items.Objects[i]).ID;
                           // If order ID is same as the first selected order, then skip it
                           if thisOrderID = firstSelectedID then
                                Continue
                           else
                               begin
                               compareDxRec := TBADxRecord.Create;
                               InitializeNewDxRec(compareDxRec);
                               GetBADxListForOrder(compareDxRec, thisOrderID);

                               thatStringList.Add(compareDxRec.FBADxCode);
                               thatStringList.Add(compareDxRec.FBASecDx1);
                               thatStringList.Add(compareDxRec.FBASecDx2);
                               thatStringList.Add(compareDxRec.FBASecDx3);

                               if DiagnosesMatch(thisStringList, thatStringList) then
                                   Result := true;
                               end;
                           end;
                    end;
  end; //case

  if Assigned(thisStringList) then
     thisStringList.Free;

  if Assigned(thatStringList) then
     thatStringList.Free;
end;

procedure GetBADxListForOrder(var thisRetVal: TBADxRecord; thisOrderID: string);
var
  i: integer;
  thisRec: TBADxRecord;
begin
   if UBAGlobals.tempDxNodeExists(thisOrderID) then
     begin
     thisRec := TBADxRecord.Create;

     if Assigned(tempDxList) then
          for i := 0 to (tempDxList.Count - 1) do
             begin
              thisRec := TBADxRecord(tempDxList.Items[i]);

              if Assigned(thisRec) then
                if (thisRec.FOrderID = thisOrderID) then
                   begin
                   with thisRetVal do
                       begin
                          FOrderID := thisRec.FOrderID;
                          FBADxCode := StringReplace(thisrec.FBADxCode,'^',':',[rfReplaceAll]);
                          FBASecDx1 := StringReplace(thisrec.FBASecDx1,'^',':',[rfReplaceAll]);
                          FBASecDx2 := StringReplace(thisrec.FBASecDx2,'^',':',[rfReplaceAll]);;
                          FBASecDx3 := StringReplace(thisrec.FBASecDx3,'^',':',[rfReplaceAll]);
                       end;
                   end;
              end;
     end;
end;

procedure DestroyDxList;
var
 i: integer;
begin
  if Assigned(tempDxList) then
     for i := 0 to pred(UBAGlobals.tempDxList.Count) do
        TObject(tempDxList[i]).Free;

     tempDxList.Clear;
     tempDxList.Free;
end;

procedure SimpleAddTempDxList(thisOrderID: string);
var
   tempDxRec: TBADxRecord;
begin
     frmBALocalDiagnoses.LoadTempRec(tempDxRec, thisOrderID);
     UBAGlobals.tempDxList.Add(TBADxRecord(tempDxRec));
end;

procedure SetBADxList;
var
  i: smallint;
begin
  if not Assigned(UBAGlobals.tempDxList) then
     begin
     UBAGlobals.tempDxList := TList.Create;
     UBAGlobals.tempDxList.Count := 0;
     end
  else
     begin
     //Kill the old Dx list
     for i := 0 to pred(UBAGlobals.tempDxList.Count) do
        TObject(UBAGlobals.tempDxList[i]).Free;

     UBAGlobals.tempDxList.Clear;

     //Create new Dx list for newly selected patient
      if not Assigned(UBAGlobals.tempDxList) then
         begin
         UBAGlobals.tempDxList := TList.Create;
         UBAGlobals.tempDxList.Count := 0;
         end;
     end;
end;

procedure SetBADxListForOrder(thisRec: TBADxRecord; thisOrderID: string);
var
  i: integer;
  foundRec: TBADxRecord;
begin
   if UBAGlobals.tempDxNodeExists(thisOrderID) then
     begin
     foundRec := TBADxRecord.Create;

     if Assigned(tempDxList) then
          for i := 0 to (tempDxList.Count - 1) do
             begin
              foundRec := TBADxRecord(tempDxList.Items[i]);

              if Assigned(thisRec) then
                if (thisOrderID = foundRec.FOrderID) then
                   begin
                   with foundRec do
                       begin
                       FOrderID := thisRec.FOrderID;
                       FBADxCode := thisRec.FBADxCode;
                       FBASecDx1 := thisRec.FBASecDx1;
                       FBASecDx2 := thisRec.FBASecDx2;
                       FBASecDx3 := thisRec.FBASecDx3;
                       end;
                   Break;
                   end;
              end;
     end;
end;

function SecondaryDxFull(thisOrderID: string) : boolean;
var
  i: integer;
  thisRec: TBADxRecord;
begin
  Result := false;

  thisRec := TBADxRecord.Create;
     for i := 0 to tempDxList.Count - 1 do
        begin
        thisRec := TBADxRecord(tempDxList.Items[i]);

        if Assigned(thisRec) then
            if thisRec.FOrderID = thisOrderID then
               begin
               if (thisRec.FBADxCode <> UBAConst.DXREC_INIT_FIELD_VAL) then
                  if (thisRec.FBASecDx1 <> UBAConst.DXREC_INIT_FIELD_VAL) then
                     if (thisRec.FBASecDx2 <> UBAConst.DXREC_INIT_FIELD_VAL) then
                        if (thisRec.FBASecDx3 <> UBAConst.DXREC_INIT_FIELD_VAL) then
                           Result := true;
               end;
        end;
end;

procedure AddSecondaryDx(thisOrderID: string; thisDxCode: string);
// Add a Secondary Dx to the first open slot in DxRec, if there IS an open slot
var
  thisRec: TBADxRecord;
  i: integer;
begin
  thisRec := TBADxRecord.Create;

  for i := 0 to tempDxList.Count - 1 do
     begin
     thisRec := TBADxRecord(tempDxList.Items[i]);

     if thisRec.FOrderID = thisOrderID then
        begin
        if (thisRec.FBASecDx1 = UBAConst.DXREC_INIT_FIELD_VAL) then
           thisRec.FBASecDx1 := thisDxCode
        else
           if (thisRec.FBASecDx2 = UBAConst.DXREC_INIT_FIELD_VAL) then
              thisRec.FBASecDx2 := thisDxCode
        else
           if (thisRec.FBASecDx3 = UBAConst.DXREC_INIT_FIELD_VAL) then
              thisRec.FBASecDx3 := thisDxCode;
        end
     end;
end;

procedure InitializeNewDxRec(var thisDxRec: TBADxRecord);
begin
  with thisDxRec do
     begin
     FExistingRecordID := UBAConst.DXREC_INIT_FIELD_VAL;
     FOrderID := UBAConst.DXREC_INIT_FIELD_VAL;
     FBADxCode := UBAConst.DXREC_INIT_FIELD_VAL;
     FBASecDx1 := UBAConst.DXREC_INIT_FIELD_VAL;
     FBASecDx2 := UBAConst.DXREC_INIT_FIELD_VAL;
     FBASecDx3 := UBAConst.DXREC_INIT_FIELD_VAL;
     end;
end;

constructor TBAGlobals.Create;
begin
  inherited Create;
end;
// This procedure is called from uPCE.pas only --  do not delete.....
procedure TBAGlobals.AddBAPCEDiag(DiagStr:string);
begin
  if (BAPCEDiagList.Count <= 0) then
    BAPCEDiagList.Add('^***Encounter Diagnoses***');
  BAPCEDiagList.Add(DiagStr);
end;

procedure TBAGlobals.ClearBAPCEDiagList;
begin
   if Assigned(BAPCEDiagList) then
      BAPCEDiagList.Free;
   BAPCEDiagList := TStringList.Create;
   BAPCEDiagList.Clear;
end;

procedure TBAGlobals.SaveBillingData;
var
  RecsToSave: TStringList;
begin
  RecsToSave := TStringList.Create;
  RecsToSave.Clear;
  RecsToSave := AttachDxToOrderList; //call with new Biling data, return-code returned
  CallV('ORWDBA1 RCVORCI',[RecsToSave]);
end;

constructor TBAPLRec.Create;
begin
  inherited Create;

end;

function TBAPLRec.BuildBAProblemRecHDR: TStringList;
// StringList used to store DX Codes selected from Encounter Form
var
  BAVisitStr   : string;
  BAFileCat    : char;
  BAFileDateTimeStr: string;
  BAProviderStr, BAProviderName : string;
begin
  BAProbFile := TStringList.Create;
  PLlist     := TStringList.Create;
  BAFileCat := Encounter.VisitCategory;
  BAFileDateTimeStr := floattostr(FMNow);
  BALocation := Encounter.Location;
  BAProviderStr := IntToStr(Encounter.Provider);
  BAProviderName := Encounter.ProviderName;

  //  Build Problem List Record, from SignOrders.
  BAVisitStr := IntToStr(BALocation) + ';' + BAFileDateTimeStr + ';' + BAFileCat;
  PLlist.Add('HDR^' + '0'{FBAInpatient} + U + U + BAVisitStr);
  PLlist.Add('VST^DT^' + BAFileDateTimeStr);
  PLlist.Add('VST^PT^' + Patient.DFN);

  if (BALocation > 0) then
    PLlist.Add('VST^HL^' + IntToStr(BALocation));

  PLlist.Add('VST^VC^' + BAFileCat);
  PLlist.Add('PRV^'+ BAProviderStr + U + U + U + BAProviderName + U + '1');
  Result := PLlist;

end;

function TBAPLRec.BuildBAProblemRecDTL(PassList: TStringList; DXSection,DXCode:string): TStringList;
var
  BAProviderStr : string;
begin
    PLlist := PassList;
    BAProviderStr := IntToStr(Encounter.Provider);
    PLlist.Add('POV+^' + Piece(DXCode,U,1) + U + Piece(DXSection,U,2) + U +
           Piece(DXCode,U,2) + U + '1' {BOOLCHAR(PRIMARY)} + U + BAProviderStr + U + '1' + U + U + U);
    Result := PLlist;
end;

//  Save New Problem record to file
function TBAPLRec.SaveProblemRec(BAProblemList:TStringList):Boolean;
begin
 Result := (sCallV('ORWPCE SAVE', [BAProblemList, 0, BALocation]) = '1');
end;

function TBAPLRec.FMToDateTime(FMDateTime: string): TDateTime;
var
  x, Year: string;
begin
  { Note: TDateTime cannot store month only or year only dates }
  x := FMDateTime + '0000000';
  if Length(x) > 12 then x := Copy(x, 1, 12);
  if StrToInt(Copy(x, 9, 4)) > 2359 then x := Copy(x,1,7) + '.2359';
  Year := IntToStr(17 + StrToInt(Copy(x,1,1))) + Copy(x,2,2);
  x := Copy(x,4,2) + '/' + Copy(x,6,2) + '/' + Year + ' ' + Copy(x,9,2) + ':' + Copy(x,11,2);
  Result := StrToDateTime(x);
end;

{-------------------------- TPLPt Class ----------------------}
constructor TBAPLPT.Create(Alist:TStringList);
var
  i: integer;
begin
  for i := 0 to AList.Count - 1 do
    case i of
      0: PtVAMC             := Copy(Alist[i],1,999);
      1: PtDead             := AList[i];
      2: PtServiceConnected := (AList[i] = '1');
      3: PtAgentOrange      := (AList[i] = '1');
      4: PtRadiation        := (AList[i] = '1');
      5: PtEnvironmental    := (AList[i] = '1');
      6: PtBID              := Alist[i];
      7: PtHNC              := (AList[i] = '1');
      8: PtMST              := (AList[i] = '1');
    end;
end;

function TBAPLPt.GetGMPDFN(dfn:string;name:string):string;
begin
  Result := dfn + u + name + u + PtBID + u + PtDead
end;

procedure TBAPLPt.LoadPatientParams(AList:TstringList);
begin
  AList.Assign(InitPt(Patient.DFN)) ;
  BAPLPt := TBAPLPt.create(Alist);
end;

function TBAPLPt.InitPt(const PatientDFN: string): TStrings ;  //*DFN*
begin
   CallV('ORQQPL INIT PT',[PatientDFN]);
   Result := RPCBrokerV.Results ;
end ;

function tempDxNodeExists(thisOrderID: string) : boolean;
// Returns true if a node with the specified Order ID exists, false otherwise.
var
  i: integer;
  thisRec: TBADxRecord;
begin
  Result := false;
  thisRec := TBADxRecord.Create;

   if Assigned(tempDxList) then
        for i := 0 to (tempDxList.Count - 1) do
           begin
            thisRec := TBADxRecord(tempDxList.Items[i]);

            if Assigned(thisRec) then
              if (thisRec.FOrderID = thisOrderID) then
                 begin
                 Result := true;
                 Break;
                 end;
            end;
end;

function BADxEntered:boolean;
var
  i,r, orderStatus: integer;
  x: string;
  passList: TStringList;
  returnList: TStringList;
  holdOrderList: TStringList;
  updatedOrderList: TStringList;
  BAUpdated : boolean;
  thisOrderID: string;
  thisRec: string;
begin
   Result := FALSE;
   BAUpdated := FALSE;
   passList:= TStringList.Create;
   returnList := TStringList.Create;
   updatedOrderList := TStringList.Create;
   updatedOrderList.Clear;
   holdOrderList := TStringList.Create;
   holdOrderList.Clear;
   // 1st.  do orders require a dx (lrmp- only)
   // if NO then continue
   // if YES, check BADxList for orders with DX enteries.
   // if ok then create data string pass to M via RPC
   orderStatus := 0;
   Result := true;

   for i := 0 to BAOrderList.Count-1 do
   begin
      thisRec := BAOrderList.Strings[i];
      thisOrderID := piece(thisRec,';',1) + ';1';  //rebuild orderID pass to M.

      x := BAOrderList.Strings[i];
      orderStatus := StrToInt(CharAt(Piece(x, ';', 2), 1));  //  Order Status 1=OK, 2=DISCONTINUE
      if orderStatus = BAOK2SIGN then
      begin
         passList.Add(piece(x,';',1));
         holdOrderList.Add(x);//  place holder for orders that can be signed!
      end;
   end;
   BAOrderList := holdOrderList; //assign signable orders to BAOrderList for further processing
     //call with passList determine if LRMP
   tCallV(returnList,'ORWDBA1 ORPKGTYP',[passList]);

  //Remove NON LRMP orders from the mix(for checking for dx entry);

   for r := 0 to BAOrderList.Count-1 do
   begin
      x:= piece(returnList.Strings[r],'^',1);
      if x = '1'  then
      begin
         updatedOrderList.Add(BAOrderList[r]);
         BAUpdated := TRUE;
      end;
   end;
   if BAUpdated then
   begin
      BAOrderList := updatedOrderList;
   end;
   if updatedOrderList.Count <= 0 then   //  order list reflecting orders flagged as -
   begin                           // dx required.
      Result := TRUE;              //  force true, no record need DX entry
      SaveBillingDxEntered;        // save billing data for orders that may have dx enteries
      Exit;                        //to do.  clean this up... when time permitts
   end
   else
   begin
   if OrdersHaveDx then
      BAGlobals.SaveBillingData
   else
   begin
      Result := FALSE;
      Exit;
   end;
  end;
end;



// returns value used to bypass Billing Aware if needed.
//  turns off visual as well and functionality
procedure SetBillingAwareSwitch;
begin
   BASwitchON := FALSE;
   BASwitchON := (sCallV('ORWDBA1 BASTATUS', [nil]) = '1');
end ;

function OrdersHaveDx:boolean;
var
  i: integer;
  thisOrderID: string;
  thisRec: string;
  tempDxRec: TBADxRecord;
begin
     Result := TRUE;
     tempDxRec := nil;
     tempDxRec := TBADxRecord.Create;
     UBAGlobals.InitializeNewDxRec(tempDxRec);

     for i := 0 to BAOrderList.Count-1 do
       begin
          thisRec := BAOrderList.Strings[i];
          thisOrderID := piece(thisRec,';',1) + ';1';  //rebuild orderID pass to M.
          if not tempDxNodeExists(thisOrderID) then
             begin
                Result := FALSE;
                Break;
             end
             else
             begin
                GetBADxListForOrder(tempDxRec, thisOrderID);
                if tempDxRec.FBADxCode = '' then
                begin
                   Result := FALSE;
                   Break;
                end;
          end;

        end;

   if Assigned(tempDxRec) then // This line added Friday 8/8/2003
       tempDxRec.free;
end;

function AttachDxToOrderList:TStringList;
var
  i: integer;
  newBillingList: TStringList;
  baseDxRec: TBADxRecord;
  currentOrderID: string;
  currentOrderString: string;
  dxString: string;

begin
   newBillingList:= TStringList.Create;
   newBillingList.Clear;
   dxString := '';
   baseDxRec := nil;
   baseDxRec := TBADxRecord.Create;

  InitializeNewDxRec(baseDxRec);

  for i := 0 to BAOrderList.Count-1 do
     begin
        currentOrderString := BAOrderList.Strings[i];
        currentOrderID := piece(BAOrderList.Strings[i],';',1)+ ';1';
        GetBADxListForOrder(baseDxRec, currentOrderID);
        dxString := currentOrderString + '^' + piece(baseDxRec.FBADxCode,':',1) + '^' + piece(baseDxRec.FBADxCode,':',2);
        if baseDxRec.FBASecDx1 <> '' then
           dxString := dxString + '^' + piece(baseDxRec.FBASecDx1,':',1) + '^' + piece(baseDxRec.FBASecDx1,':',2);
        if baseDxRec.FBASecDx2 <> '' then
           dxString := dxString + '^' + piece(baseDxRec.FBASecDx2,':',1) + '^' + piece(baseDxRec.FBASecDx2,':',2);
        if baseDxRec.FBASecDx3 <> '' then
           dxString := dxString + '^' + piece(baseDxRec.FBASecDx3,':',1) + '^' + piece(baseDxRec.FBASecDx3,':',2);

        NewBillingList.Add(dxString);
     end;
     Result := NewBillingList;
end;


procedure SaveBillingDxEntered;  // if not mandatory and user enters dx.
var
 ordersWithDx: integer;
 newBillingList: TStringList;
  baseDxRec, tempDxRec: TBADxRecord;
 currentOrderID, thisOrderID: string;
 currentOrderString, thisRec: string;
 i : integer;
begin
// verify Dx has been entered for orders checked for signature..
     ordersWithDx := 0;
     tempDxRec := TBADxRecord.Create;
     UBAGlobals.InitializeNewDxRec(tempDxRec);
     for i := 0 to BAOrderList.Count-1 do
     begin
        thisRec := BAOrderList.Strings[i];
        thisOrderID := piece(thisRec,';',1) + ';1';  //rebuild orderID pass to M.
        if tempDxNodeExists(thisOrderID) then
           inc(ordersWithDx);
     end;

     // if orders have dx enteries - save billing data.
     if ordersWithDx > 0 then
     begin
        newBillingList:= TStringList.Create;
        newBillingList.Clear;
        baseDxRec := nil;
        baseDxRec := TBADxRecord.Create;
       InitializeNewDxRec(baseDxRec);
       for i := 0 to BAOrderList.Count-1 do
       begin
          currentOrderString := BAOrderList.Strings[i];
          currentOrderID := piece(BAOrderList.Strings[i],';',1)+ ';1';
          GetBADxListForOrder(baseDxRec, currentOrderID);
          if baseDxRec.FBADxCode <> '' then
          begin
             NewBillingList.Add(currentOrderString +'^'+ baseDxRec.FBADxCode +'^'+ baseDxRec.FBASecDx1+
                             '^'+ baseDxRec.FBASecDx2+'^'+ baseDxRec.FBASecDx3);
          end;
       end;
   if NewBillingList.Count > 0 then
      CallV('ORWDBA1 RCVORCI',[NewBillingList]);
   if Assigned(NewBillingList) then NewBillingList.free;

  end;
end;

end.

