unit fARTAllgy;

interface

uses
  SysUtils, WinTypes, WinProcs, Messages, Classes, Graphics, Controls,
  Forms, Dialogs, StdCtrls, ORCtrls, ORfn, ExtCtrls, ComCtrls, uConst,
  Menus, ORDtTm, Buttons, fODBase, fAutoSz, fOMAction, rODAllergy;

type
  TfrmARTAllergy = class(TfrmOMAction)
    pnlBase: TORAutoPanel;
    cmdAccept: TButton;
    cmdQuit: TButton;
    pgAllergy: TPageControl;
    tabGeneral: TTabSheet;
    tabSignsSymptoms: TTabSheet;
    tabObservations: TTabSheet;
    tabComments: TTabSheet;
    tabVerify: TTabSheet;
    lblSymptoms: TOROffsetLabel;
    cboSymptoms: TORComboBox;
    lblSelectedSymptoms: TOROffsetLabel;
    btnDateTime: TButton;
    btnRemove: TButton;
    lstSelectedSymptoms: TORListBox;
    ckNoKnownAllergies: TCheckBox;
    btnCurrent: TButton;
    lblAgent: TOROffsetLabel;
    lstAllergy: TORListBox;
    btnAgent: TSpeedButton;
    lblReactionType: TOROffsetLabel;
    cboReactionType: TORComboBox;
    lblOriginator: TOROffsetLabel;
    cboOriginator: TORComboBox;
    lblOriginateDate: TOROffsetLabel;
    calOriginated: TORDateBox;
    ckSignOff: TCheckBox;
    ckEnteredInError: TCheckBox;
    ckChartMarked: TCheckBox;
    ckIDBand: TCheckBox;
    lblVerifier: TOROffsetLabel;
    ckVerified: TCheckBox;
    cboVerifier: TORComboBox;
    calVerifyDate: TORDateBox;
    lblVerifyDate: TOROffsetLabel;
    cmdPrevCmts: TButton;
    lblComments: TOROffsetLabel;
    memComments: TRichEdit;
    cmdPrevObs: TButton;
    grpObsHist: TRadioGroup;
    lblObservedDate: TOROffsetLabel;
    calObservedDate: TORDateBox;
    lblSeverity: TOROffsetLabel;
    cboSeverity: TORComboBox;
    dlgReactionDateTime: TORDateTimeDlg;
    procedure btnAgentClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure cboOriginatorNeedData(Sender: TObject; const StartFrom: String;
      Direction, InsertAt: Integer);
    procedure cboSymptomsNeedData(Sender: TObject; const StartFrom: String;
      Direction, InsertAt: Integer);
    procedure lstAllergySelect(Sender: TObject);
    procedure grpObsHistClick(Sender: TObject);
    procedure ControlChange(Sender: TObject);
    procedure memCommentsExit(Sender: TObject);
    procedure cboSymptomsClick(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure ckNoKnownAllergiesClick(Sender: TObject);
    procedure EnableControls(EnabledStatus: boolean);
    procedure cmdAcceptClick(Sender: TObject);
    procedure btnCurrentClick(Sender: TObject);
    procedure btnRemoveClick(Sender: TObject);
    procedure lstAllergyClick(Sender: TObject);
    procedure btnDateTimeClick(Sender: TObject);
    procedure cboSymptomsMouseClick(Sender: TObject);
    procedure cboSymptomsKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure cmdQuitClick(Sender: TObject);
    procedure cmdPrevCmtsClick(Sender: TObject);
    procedure cmdPrevObsClick(Sender: TObject);
    procedure lstSelectedSymptomsChange(Sender: TObject);
    procedure cboVerifierNeedData(Sender: TObject; const StartFrom: String;
      Direction, InsertAt: Integer);
  private
    FLastAllergyID: string;
    FEditAllergyIEN: integer;
    FNKAOrder: boolean;
    FChanged: Boolean;
  protected
    procedure InitDialog;
    procedure Validate(var AnErrMsg: string);
    function  ValidSave: Boolean;
    procedure SetupDialog;
    procedure SetupVerifyFields(ARec: TAllergyRec);
  end;

function EnterEditAllergy(AllergyIEN: integer; AddNew: boolean): boolean;

var
  AllergyList: TStringList;
  OldRec, NewRec: TAllergyRec;
  Defaults: TStringList;
  Changing: Boolean;
  uEditing: Boolean;
  uUserCanVerify: boolean;

implementation

{$R *.DFM}

uses
  rODBase, uCore, rCore, rCover, fAllgyFind, fPtCWAD, fRptBox;

const
  TX_NO_ALLERGY       = 'An allergy must be specified.'    ;
  TX_NO_REACTION      = 'A reaction type must be entered for this allergy.'  ;
  TX_NO_SYMPTOMS      = 'Symptoms must be selected for this observed allergy and reaction.';
  TX_NO_OBSERVER      = 'An observer must be selected for this allergy and reaction .';
  TX_NO_FUTURE_DATES  = 'Dates in the future are not allowed.';
  TX_BAD_DATE         = 'Dates must be in the format m/d/y or m/y or y, or T-d.';
  TX_CAP_FUTURE       = 'Invalid date';
  TX_NO_SAVE     = 'This item cannot be saved for the following reason(s):' + CRLF + CRLF;
  TX_NO_SAVE_CAP = 'Unable to Save Allergy/Reaction';
  TX_SAVE_ERR    = 'Unexpected error - it was not possible to save this request.';

function EnterEditAllergy(AllergyIEN: integer; AddNew: boolean): boolean;
var
  frmARTAllergy: TfrmARTAllergy;
begin
  uEditing := not AddNew;
  Result := False;
  frmARTAllergy := TfrmARTAllergy.Create(Application);
  with frmARTAllergy do
    try
      ResizeFormToFont(TForm(frmARTAllergy));
      FChanged     := False;
      if uEditing then
        begin
          frmARTAllergy.Caption := 'Edit Allergy or Adverse Reaction';
          FEditAllergyIEN := AllergyIEN;
          if FEditAllergyIEN = 0 then exit;
          StatusText('Loading Allergy for Edit');
          OldRec := LoadAllergyForEdit(FEditAllergyIEN);
          NewRec.IEN := OldRec.IEN;
          SetupDialog;
        end;
      StatusText('');
      ShowModal;
      Result := FChanged;
    finally
      frmARTAllergy.Release;
    end;
end;

procedure TfrmARTAllergy.FormCreate(Sender: TObject);
begin
  inherited;       // what to do here?  How to set up dialog defaults without order dialog to supply prompts?
  AllergyList := TStringList.Create;
  FillChar(OldRec, SizeOf(OldRec), 0);
  FillChar(NewRec, SizeOf(NewRec), 0);
  with NewRec do
    begin
      SignsSymptoms  := TStringList.Create ;
      IDBandMarked   := TStringList.Create;
      ChartMarked    := TStringList.Create;
      Observations   := TStringList.Create;
      Comments       := TStringList.Create ;
      NewComments    := TStringList.Create ;
    end;
  Defaults    := TStringList.Create;
  //StatusText('Loading Dialog Definition');
  //Responses.Dialog := 'GMRAOR ALLERGY ENTER/EDIT';   // loads formatting info     // **** PROBLEM!!  ****************
  StatusText('Loading Default Values');
  uUserCanVerify := HasSecurityKey('GMRA-ALLERGY VERIFY');
  Defaults.Assign(ODForAllergies);  // returns TStrings with defaults      // **** OK HERE    ****************
  StatusText('Initializing Long List');
  ExtractItems(cboSymptoms.MItems, Defaults, 'Top Ten');
  cboSymptoms.InsertSeparator;
  cboOriginator.InitLongList(User.Name) ;
  cboOriginator.SelectByIEN(User.DUZ);
  //PreserveControl(cboSymptoms);
  //PreserveControl(cboOriginator);
  InitDialog;
  if not uEditing then
    begin
      SetupVerifyFields(NewRec);
      btnAgentClick(Self);
    end
  else
    SetupVerifyFields(OldRec);
end;

procedure TfrmARTAllergy.InitDialog;
begin
  inherited;
  Changing := True;
  ExtractItems(cboReactionType.MItems, Defaults, 'Reactions');
  ExtractItems(cboSeverity.MItems, Defaults, 'Severity');
  lstAllergy.MItems.Add('-1^Click button to search ---->');
  grpObsHist.ItemIndex := 1;
  calObservedDate.Text := '';
  cboSeverity.ItemIndex := -1;
  cboSymptoms.ItemIndex := -1;
  memComments.Clear;
  cmdPrevCmts.Enabled := (uEditing and (OldRec.Comments <> nil) and (OldRec.Comments.Text <> ''));
  cmdPrevObs.Enabled := (uEditing and (OldRec.Comments <> nil) and (OldRec.Observations.Text <> ''));
  ListAllergies(AllergyList);
  with AllergyList do
    if Count > 0 then
      begin
        if Piece(Strings[0], U, 1) = '' then
          ckNoKnownAllergies.Enabled := True
        else
          begin
            ckNoKnownAllergies.Enabled := False;
            btnCurrent.Enabled := True;
          end;
      end
    else
      begin
        btnCurrent.Enabled := False;
        ckNoKnownAllergies.Enabled := True;
      end;
  StatusText('');
  //memOrder.Clear ;
  Changing := False;
end;

procedure TfrmARTAllergy.SetupDialog;
begin
  if uEditing then with OldRec do
    begin
      Changing := True;
      ckNoKnownAllergies.Checked := NoKnownAllergies;
      btnAgent.Enabled := not Verified;
      lstAllergy.MItems.Clear;
      lstAllergy.MItems.Insert(0, U + CausativeAgent);
      lstAllergy.ItemIndex := 0;
      lstAllergySelect(Self);
      cboReactionType.SelectByID(Piece(NatureOfReaction, U, 1));
      lstSelectedSymptoms.MItems.Assign(SignsSymptoms);
      calOriginated.FMDateTime := Originated;
      cboOriginator.InitLongList(OriginatorName);
      cboOriginator.SelectByIEN(Originator);
      ckSignOff.Checked := SignOff;
      { TODO -oRich V. -cART/Allergy : Change to calendar entry fields and prior entries button. }
      //ckIDBand.Checked := IDBandMarked;
      //ckChartMarked.Checked := ChartMarked;
      ckEnteredInError.Checked := EnteredInError;
      if Piece(Observed_Historical, U, 1) <> '' then
        case UpperCase(Piece(Observed_Historical, U, 1))[1] of
          'O': grpObsHist.ItemIndex := 0;
          'H': grpObsHist.ItemIndex := 1;
        end
      else grpObsHist.ItemIndex := -1;
      calObservedDate.FMDateTime := ReactionDate;
      cmdPrevObs.Enabled := (OldRec.Observations.Text <> '');
      cboSeverity.SelectByID(Piece(Severity, U, 1));
      cmdPrevCmts.Enabled := Comments.Text <> '';
      Changing := False;
      ControlChange(Self);
    end;
end;

procedure TfrmARTAllergy.Validate(var AnErrMsg: string);
var
  tmpDate: TFMDateTime;
(*const
  TX_NO_LOCATION = 'A location must be identified.' + CRLF +
                   '(Select File | Update Provider/Location)';
  TX_NO_PROVIDER = 'A provider who is authorized to write orders must be identified.' + CRLF +
                   '(Select File | Update Provider/Location)';*)

  procedure SetError(const x: string);
  begin
    if Length(AnErrMsg) > 0 then AnErrMsg := AnErrMsg + CRLF;
    AnErrMsg := AnErrMsg + x;
  end;

begin
  AnErrMsg := '';
  if not ckNoKnownAllergies.Checked then
    begin
      if lstAllergy.Items.Count = 0 then SetError(TX_NO_ALLERGY)
      else if (Length(lstAllergy.DisplayText[0]) = 0) or
         (Piece(lstAllergy.MItems[0], U, 1) = '-1') then SetError(TX_NO_ALLERGY);
      if (grpObsHist.ItemIndex = 0) and (lstSelectedSymptoms.Items.Count = 0)   then SetError(TX_NO_SYMPTOMS);
      if cboReactionType.ItemID = ''           then SetError(TX_NO_REACTION);
    end;
  if cboOriginator.ItemIEN = 0             then SetError(TX_NO_OBSERVER);
  if calObservedDate.Text <> '' then
    begin
      tmpDate := ValidDateTimeStr(calObservedDate.Text, 'TS');
      if tmpDate > FMNow then  SetError(TX_NO_FUTURE_DATES);
      if tmpDate < 0 then SetError(TX_BAD_DATE);
    end;
(*  if Encounter.Location = 0 then AnErrMsg := TX_NO_LOCATION;
  if (Encounter.Provider = 0) or (PersonHasKey(Encounter.Provider, 'PROVIDER') = False)
    then AnErrMsg := TX_NO_PROVIDER;*)
end;

procedure TfrmARTAllergy.cboOriginatorNeedData(Sender: TObject;
  const StartFrom: string; Direction, InsertAt: Integer);
begin
  inherited;
  cboOriginator.ForDataUse(SubSetOfPersons(StartFrom, Direction));
end;

procedure TfrmARTAllergy.cboSymptomsNeedData(Sender: TObject;
  const StartFrom: string; Direction, InsertAt: Integer);
begin
  inherited;
  cboSymptoms.ForDataUse(SubSetOfSymptoms(StartFrom, Direction));
end;

procedure TfrmARTAllergy.grpObsHistClick(Sender: TObject);
begin
  inherited;
  Changing := True;
  cboSeverity.ItemIndex := -1;
  case grpObsHist.ItemIndex of
    0:  begin
          cboSeverity.Visible := True;
          lblSeverity.Visible := True;
        end;
    1:  begin
          cboSeverity.Visible := False;
          lblSeverity.Visible := False;
        end;
  end;
  Changing := False;
  ControlChange(Self);
end;

procedure TfrmARTAllergy.ControlChange(Sender: TObject);
var
  tmpDate: TFMDateTime;
begin
  inherited;
  if Changing then Exit;
  with NewRec do
    begin
      if ckNoKnownAllergies.Checked then
        begin
          with cboOriginator do if ItemIEN > 0 then
            begin
              Originator := ItemIEN;
              OriginatorName := Text;
            end;
          with calOriginated do
            begin
              tmpDate := ValidDateTimeStr(calOriginated.Text, 'TS');
              if tmpDate > 0 then Originated := tmpDate;
            end;
          NoKnownAllergies := True;
        end
      else
        with lstAllergy do if (Items.Count > 0) then
          if (Piece(MItems[0], U, 1) <>  '-1') and (Length(DisplayText[0]) > 0) then
            begin
              CausativeAgent := Trim(Piece(DisplayText[0], '<', 1));
              with cboReactionType do if ItemID <> '' then NatureOfReaction := ItemID + U + Text;
              with cboOriginator do if ItemIEN > 0 then
                begin
                  Originator := ItemIEN;
                  OriginatorName := Text;
                end;
              tmpDate := ValidDateTimeStr(calOriginated.Text, 'TS');
              if tmpDate > 0 then Originated := tmpDate;
              SignsSymptoms.Assign(lstSelectedSymptoms.MItems);
              Verified := ckVerified.Checked;
              with cboVerifier do if ItemIEN > 0 then
                begin
                  Verifier := ItemIEN;
                  VerifierName := Text;
                end;
              tmpDate := ValidDateTimeStr(calVerifyDate.Text, 'TS');
              if tmpDate > 0 then VerifiedDateTime := tmpDate;
              SignOff := ckSignOff.Checked;
              { TODO -oRich V. -cART/Allergy : Pass back prior entries, or only new ones? }
              NewRec.ChartMarked.Clear;
              if ckChartMarked.Checked then ChartMarked.Add(FloatToStr(FMNow));
              //  else NewRec.ChartMarked.Assign(OldRec.ChartMarked);    // clear????
              NewRec.IDBandMarked.Clear;
              if ckIDBand.Checked then IDBandMarked.Add(FloatToStr(FMNow));
              //  else NewRec.IDBandMarked.Assign(OldRec.IDBandMarked);  // clear????
              EnteredInError := ckEnteredInError.Checked;
              if EnteredInError then
                begin
                  DateEnteredInError := FMNow;
                  UserEnterIngInError := User.DUZ;
                end;
              with grpObsHist do if ItemIndex   > -1 then
                begin
                  if ItemIndex = 0 then Observed_Historical := 'o^OBSERVED'
                  else Observed_Historical := 'h^HISTORICAL';
                end;
              tmpDate := ValidDateTimeStr(calObservedDate.Text, 'TS');
              if tmpDate > 0 then ReactionDate := tmpDate;
              with cboSeverity do if ItemID <> '' then Severity := ItemID;
              with memComments do if GetTextLen > 0 then NewComments.Assign(Lines);
            end;
    end;
end;

procedure TfrmARTAllergy.lstAllergySelect(Sender: TObject);
begin
  inherited;
  with lstAllergy do
    begin
      if Items.Count = 0 then
        Exit
      else if Piece(MItems[0], U, 1) = '-1' then
        Exit;
      if Piece(MItems[0], U, 1) <> FLastAllergyID then FLastAllergyID := Piece(MItems[0], U, 1) else Exit;
      Changing := True;
      //if Sender <> Self then FillChar(NewRec, SizeOf(NewRec), 0);       // Sender=Self when called from SetupDialog
      Changing := False;
    end;
  ControlChange(Self) ;
end;

procedure TfrmARTAllergy.memCommentsExit(Sender: TObject);
var
  AStringList: TStringList;
begin
  inherited;
  AStringList := TStringList.Create;
  try
    AStringList.Assign(memComments.Lines);
    LimitStringLength(AStringList, 74);
    memComments.Lines.Assign(AstringList);
    ControlChange(Self);
  finally
    AStringList.Free;
  end;
end;

procedure TfrmARTAllergy.btnAgentClick(Sender: TObject);
var
  Allergy: string;
begin
  inherited;
  AllergyLookup(Allergy, ckNoKnownAllergies.Enabled);
  if Piece(Allergy, U, 1) = '-1' then
    ckNoKnownAllergies.Checked := True
  else
    if Allergy <> '' then
      begin
        lstAllergy.Clear;
        lstAllergy.MItems.Add(Allergy);
        cboReactionType.SelectByID(Piece(Allergy, U, 4));
      end;
  ControlChange(lstAllergy);
end;

procedure TfrmARTAllergy.cboSymptomsClick(Sender: TObject);
begin
  inherited;
  if cboSymptoms.ItemIndex < 0 then exit;
  Changing := True;
  if lstSelectedSymptoms.SelectByID(cboSymptoms.ItemID) > -1 then exit;
  with lstSelectedSymptoms do
    begin
      MItems.Add(cboSymptoms.MItems[cboSymptoms.ItemIndex]);
      SelectByID(cboSymptoms.ItemID);
    end;
  Changing := False;
  ControlChange(Self)
end;

procedure TfrmARTAllergy.FormDestroy(Sender: TObject);
begin
  OldRec.SignsSymptoms.Free;
  OldRec.IDBandMarked.Free;
  OldRec.ChartMarked.Free;
  OldRec.Observations.Free;
  OldRec.Comments.Free;
  OldRec.NewComments.Free;
  NewRec.SignsSymptoms.Free;
  NewRec.IDBandMarked.Free;
  NewRec.ChartMarked.Free;
  NewRec.Observations.Free;
  NewRec.Comments.Free;
  NewRec.NewComments.Free;
  Defaults.Free;
  AllergyList.Free;
  inherited;
end;

procedure TfrmARTAllergy.ckNoKnownAllergiesClick(Sender: TObject);
begin
  inherited;
  FNKAOrder := ckNoKnownAllergies.Checked;
  EnableControls(not FNKAOrder);
  ControlChange(Self);
end;

procedure TfrmARTAllergy.EnableControls(EnabledStatus: boolean);
begin
   InitDialog;
   btnAgent.Enabled            := EnabledStatus;
   cboReactionType.Enabled     := EnabledStatus;
   lblReactionType.Enabled     := EnabledStatus;
   lblAgent.Enabled            := EnabledStatus;
   lblSymptoms.Enabled         := EnabledStatus;
   lblSelectedSymptoms.Enabled := EnabledStatus;
   grpObsHist.Enabled          := EnabledStatus;
   memComments.Enabled         := EnabledStatus;
   lblComments.Enabled         := EnabledStatus;
   lstSelectedSymptoms.Enabled := EnabledStatus;
   lblObservedDate.Enabled     := EnabledStatus;
   calObservedDate.Enabled     := EnabledStatus;
   lblSeverity.Enabled         := EnabledStatus;
   cboSeverity.Enabled         := EnabledStatus;
   lstAllergy.Enabled          := EnabledStatus;
   cboSymptoms.Enabled         := EnabledStatus;
   btnDateTime.Enabled         := EnabledStatus;
end;

procedure TfrmARTAllergy.cmdAcceptClick(Sender: TObject);
var
  Saved: string;
begin
  if ValidSave then
    begin
      Saved := OldSaveAllergy(NewRec);
      //Saved := NewSaveAllergy(NewRec);
      FChanged := (Piece(Saved, U, 1) = '0');
      if not FChanged then
        InfoBox(TX_NO_SAVE + Piece(Saved, U, 2), TX_NO_SAVE_CAP, MB_OK);
      Close;
    end;
end;

function TfrmARTAllergy.ValidSave: Boolean;
var
  ErrMsg: string;
begin
  Result := True;
  Validate(ErrMsg);
  if Length(ErrMsg) > 0 then
  begin
    InfoBox(TX_NO_SAVE + ErrMsg, TX_NO_SAVE_CAP, MB_OK);
    Result := False;
  end;
end;

procedure TfrmARTAllergy.btnCurrentClick(Sender: TObject);
const
  VIEW_CURRENT = 'Current Allergies for ';
begin
  inherited;
  ReportBox(DetailPosting('A'), VIEW_CURRENT + Patient.Name, True)
end;

procedure TfrmARTAllergy.btnRemoveClick(Sender: TObject);
var
  i: integer;
begin
  inherited;
  with lstSelectedSymptoms do
    begin
      if (Items.Count = 0) or (ItemIndex = -1) then exit;
      i := ItemIndex;
      MItems.Delete(ItemIndex);
      ItemIndex := i - 1;
      if (Items.Count > 0) and (ItemIndex = -1) then ItemIndex := 0;
    end;
end;

procedure TfrmARTAllergy.lstAllergyClick(Sender: TObject);
begin
  inherited;
  lstAllergy.ItemIndex := -1;
end;

procedure TfrmARTAllergy.btnDateTimeClick(Sender: TObject);
var
  AFMDateTime: TFMDateTime;
begin
  inherited;
  with lstSelectedSymptoms do
    begin
      if (Items.Count = 0) or (ItemIndex = -1) then exit;
      AFMDateTime := MakeFMDateTime(Piece(MItems[ItemIndex], U, 3));
      if AFMDateTime > 0 then
        dlgReactionDateTime.FMDateTime := AFMDateTime
      else
        dlgReactionDateTime.FMDateTime := FMNow;
      if not dlgReactionDateTime.Execute then exit;
      if dlgReactionDateTime.FMDateTime > FMNow then
        InfoBox(TX_NO_FUTURE_DATES, TX_CAP_FUTURE, MB_OK)
      else
        MItems[ItemIndex] := MItems[ItemIndex] + U + FloatToStr(dlgReactionDateTime.FMDateTime) + U + FormatFMDateTime('mmm dd,yyyy@hh:nn', dlgReactionDateTime.FMDateTime);
    end;
end;

procedure TfrmARTAllergy.cboSymptomsMouseClick(Sender: TObject);
begin
  inherited;
  if cboSymptoms.ItemIndex < 0 then exit;
  Changing := True;
  if lstSelectedSymptoms.SelectByID(cboSymptoms.ItemID) > -1 then exit;
  with lstSelectedSymptoms do
    begin
      MItems.Add(cboSymptoms.MItems[cboSymptoms.ItemIndex]);
      SelectByID(cboSymptoms.ItemID);
    end;
  Changing := False;
  ControlChange(Self)
end;

procedure TfrmARTAllergy.cboSymptomsKeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
  inherited;
  if Key = VK_RETURN then cboSymptomsMouseClick(Self);
end;

procedure TfrmARTAllergy.cmdQuitClick(Sender: TObject);
begin
  inherited;
  FChanged := False;
  Close;
end;

procedure TfrmARTAllergy.cmdPrevCmtsClick(Sender: TObject);
const
  CMT_CAPTION = 'View previous comments';
begin
  inherited;
  ReportBox(OldRec.Comments, CMT_CAPTION, False);
end;

procedure TfrmARTAllergy.cmdPrevObsClick(Sender: TObject);
const
  OBS_CAPTION = 'View previous observations';
begin
  inherited;
  ReportBox(OldRec.Observations, OBS_CAPTION, False);
end;

procedure TfrmARTAllergy.lstSelectedSymptomsChange(Sender: TObject);
begin
  inherited;
  with lstSelectedSymptoms do
    begin
      btnRemove.Enabled := (ItemIndex = -1) or
                           not (uEditing and
                             (OldRec.SignsSymptoms <> nil) and
                             (OldRec.SignsSymptoms.IndexOf(MItems[ItemIndex]) > -1));
      btnDateTime.Enabled := btnRemove.Enabled;
    end;
  ControlChange(Self);
end;

procedure TfrmARTAllergy.cboVerifierNeedData(Sender: TObject;
  const StartFrom: String; Direction, InsertAt: Integer);
begin
  inherited;
  cboVerifier.ForDataUse(SubSetOfPersons(StartFrom, Direction));
end;

procedure TfrmARTAllergy.SetupVerifyFields(ARec: TAllergyRec);
var
  CanBeVerified: boolean;
begin
  with ARec do
    begin
      ckVerified.Checked := Verified;
      CanBeVerified := (not Verified) and uUserCanVerify;
      ckVerified.Enabled := CanBeVerified;
      lblVerifier.Enabled := CanBeVerified;
      lblVerifyDate.Enabled := CanBeVerified;
      if CanBeVerified then
        begin
          cboVerifier.InitLongList(User.Name);
          cboVerifier.SelectByIEN(User.DUZ);
          calVerifyDate.FMDateTime := FMNow;
        end
      else
        begin
          cboVerifier.InitLongList(VerifierName);
          cboVerifier.SelectByIEN(Verifier);
          calVerifyDate.FMDateTime := VerifiedDateTime;
        end;
      cboVerifier.Enabled := CanBeVerified;
      calVerifyDate.Enabled := CanBeVerified;
    end;
end;

end.
